part of 'package:native_pdf_view/src/native_pdf_view.dart';

class SnappingPdfView extends StatefulWidget {
  const SnappingPdfView({
    @required this.controller,
    @required this.onPageChanged,
    @required this.pageLoader,
    @required this.pageBuilder,
    @required this.pageSnapping,
    @required this.scrollDirection,
    @required this.physics,
    Key key,
  }) : super(key: key);

  final PdfController controller;

  /// Called whenever the page in the center of the viewport changes
  final void Function(int page) onPageChanged;

  final Widget pageLoader;

  /// Page builder
  final PDFViewPageBuilder pageBuilder;

  final bool pageSnapping;

  /// Page turning direction
  final Axis scrollDirection;

  final ScrollPhysics physics;

  @override
  _SnappingPdfViewState createState() => _SnappingPdfViewState();
}

class _SnappingPdfViewState extends State<SnappingPdfView>
    with SingleTickerProviderStateMixin<SnappingPdfView> {
  final Map<int, PdfPageImage> _pages = {};
  AnimationController _animationController;

  @override
  void initState() {
    _animationController = AnimationController(
      duration: Duration(milliseconds: 200),
      vsync: this,
    );
    super.initState();
  }

  @override
  void dispose() {
    _animationController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Listener(
      onPointerSignal: (PointerSignalEvent event) {
        if (event is PointerScrollEvent && !widget.pageSnapping) {
          widget.controller._pageController.animateTo(
            widget.controller._pageController.offset +
                (event.scrollDelta.dy * window.devicePixelRatio),
            duration: Duration(milliseconds: 75),
            curve: Curves.ease,
          );
        }
      },
      child: ExtendedImageGesturePageView.builder(
        itemBuilder: (BuildContext context, int index) =>
            FutureBuilder<PdfPageImage>(
          future: widget.controller._getPageImage(index),
          builder: (_, snapshot) {
            if (snapshot.hasData) {
              return KeyedSubtree(
                key: Key('$runtimeType.page.'
                    '${widget.controller._document.hashCode}.'
                    '${_pages[index].pageNumber}'),
                child: widget.pageBuilder(
                  _pages[index],
                  index == 0, // FIXME:
                  _animationController,
                ),
              );
            }

            return KeyedSubtree(
              key: Key('$runtimeType.page.loading'),
              child: widget.pageLoader ?? SizedBox(),
            );
          },
        ),
        itemCount: widget.controller._document.pagesCount,
        onPageChanged: (int index) {
          // _currentIndex = index;
          widget.onPageChanged?.call(index + 1);
        },
        controller: widget.controller?._pageController,
        scrollDirection: widget.scrollDirection,
        pageSnapping: widget.pageSnapping,
        physics: widget.physics,
      ),
    );
  }

  static const List<double> _doubleTapScales = <double>[1.0, 2.0, 3.0];

  /// Default page builder
  static Widget _pageBuilder(
    PdfPageImage pageImage,
    bool isCurrentIndex,
    AnimationController animationController,
  ) {
    Animation<double> _doubleTapAnimation;
    void Function() _animationListener;

    Widget image = ExtendedImage.memory(
      pageImage.bytes,
      key: Key(pageImage.hashCode.toString()),
      fit: BoxFit.contain,
      mode: ExtendedImageMode.gesture,
      initGestureConfigHandler: (_) => GestureConfig(
        minScale: 1,
        maxScale: 3.0,
        animationMinScale: .75,
        animationMaxScale: 3.0,
        speed: 1,
        inertialSpeed: 100,
        inPageView: true,
        initialScale: 1.0,
        cacheGesture: false,
      ),
      onDoubleTap: (ExtendedImageGestureState state) {
        final pointerDownPosition = state.pointerDownPosition;
        final begin = state.gestureDetails.totalScale;
        double end;

        _doubleTapAnimation?.removeListener(_animationListener);

        animationController
          ..stop()
          ..reset();

        if (begin == _doubleTapScales[0]) {
          end = _doubleTapScales[1];
        } else {
          if (begin == _doubleTapScales[1]) {
            end = _doubleTapScales[2];
          } else {
            end = _doubleTapScales[0];
          }
        }

        _animationListener = () {
          //print(_animation.value);
          state.handleDoubleTap(
              scale: _doubleTapAnimation.value,
              doubleTapPosition: pointerDownPosition);
        };
        _doubleTapAnimation = animationController
            .drive(Tween<double>(begin: begin, end: end))
              ..addListener(_animationListener);

        animationController.forward();
      },
    );
    if (isCurrentIndex) {
      image = Hero(
        tag: 'pdf_view' + pageImage.pageNumber.toString(),
        child: image,
      );
    }
    return image;
  }
}
